@@ -72,6 +72,7 @@
 #include <dict_db.h>
 #include <dict_dbm.h>
 #include <dict_cdb.h>
+#include <dict_lmdb.h>
 #include <warn_stat.h>
 
 /* Global directory. */
@@ -99,6 +100,7 @@ static const NAME_CODE data_redirect_map_types[] = {
     DICT_TYPE_HASH, 1,
     DICT_TYPE_BTREE, 1,
     DICT_TYPE_DBM, 1,
+    DICT_TYPE_LMDB, 1,
     DICT_TYPE_CDB, 1,			/* not a read-write map type */
     "sdbm", 1,				/* legacy 3rd-party TLS */
     "dbz", 1,				/* just in case */
