@@ -151,6 +151,8 @@
 /* .ad
 /* .fi
 /*	The Secure Mailer license must be distributed with this software.
+/* HISTORY
+/*	This service was introduced with Postfix version 2.2.
 /* AUTHOR(S)
 /*	Lutz Jaenicke
 /*	BTU Cottbus
@@ -200,13 +202,13 @@
 #include <attr.h>
 #include <set_eugid.h>
 #include <htable.h>
+#include <warn_stat.h>
 
 /* Global library. */
 
 #include <mail_conf.h>
 #include <mail_params.h>
 #include <mail_version.h>
-#include <tls_mgr.h>
 #include <mail_proto.h>
 #include <data_redirect.h>
 
@@ -218,6 +220,8 @@
 /* TLS library. */
 
 #ifdef USE_TLS
+#include <tls_mgr.h>
+#define TLS_INTERNAL
 #include <tls.h>			/* TLS_MGR_SCACHE_<type> */
 #include <tls_prng.h>
 #include <tls_scache.h>
@@ -231,13 +235,13 @@ char   *var_tls_rand_source;
 int     var_tls_rand_bytes;
 int     var_tls_reseed_period;
 int     var_tls_prng_exch_period;
-int     var_smtpd_tls_loglevel;
+char   *var_smtpd_tls_loglevel;
 char   *var_smtpd_tls_scache_db;
 int     var_smtpd_tls_scache_timeout;
-int     var_smtp_tls_loglevel;
+char   *var_smtp_tls_loglevel;
 char   *var_smtp_tls_scache_db;
 int     var_smtp_tls_scache_timeout;
-int     var_lmtp_tls_loglevel;
+char   *var_lmtp_tls_loglevel;
 char   *var_lmtp_tls_scache_db;
 int     var_lmtp_tls_scache_timeout;
 char   *var_tls_rand_exch_name;
@@ -282,20 +286,26 @@ typedef struct {
     TLS_SCACHE *cache_info;		/* cache handle */
     int     cache_active;		/* cache status */
     char  **cache_db;			/* main.cf parameter value */
-    int    *cache_loglevel;		/* main.cf parameter value */
+    const char *log_param;		/* main.cf parameter name */
+    char  **log_level;			/* main.cf parameter value */
     int    *cache_timeout;		/* main.cf parameter value */
 } TLSMGR_SCACHE;
 
-TLSMGR_SCACHE cache_table[] = {
+static TLSMGR_SCACHE cache_table[] = {
     TLS_MGR_SCACHE_SMTPD, 0, 0, &var_smtpd_tls_scache_db,
+    VAR_SMTPD_TLS_LOGLEVEL,
     &var_smtpd_tls_loglevel, &var_smtpd_tls_scache_timeout,
     TLS_MGR_SCACHE_SMTP, 0, 0, &var_smtp_tls_scache_db,
+    VAR_SMTP_TLS_LOGLEVEL,
     &var_smtp_tls_loglevel, &var_smtp_tls_scache_timeout,
     TLS_MGR_SCACHE_LMTP, 0, 0, &var_lmtp_tls_scache_db,
+    VAR_LMTP_TLS_LOGLEVEL,
     &var_lmtp_tls_loglevel, &var_lmtp_tls_scache_timeout,
     0,
 };
 
+#define	smtpd_cache	(cache_table[0])
+
  /*
   * SLMs.
   */
@@ -449,6 +459,44 @@ static void tlsmgr_cache_run_event(int unused_event, char *ctx)
 			cache->cache_info->timeout);
 }
 
+/* tlsmgr_key - return matching or current RFC 5077 session ticket keys */
+
+static int tlsmgr_key(VSTRING *buffer, int timeout)
+{
+    TLS_TICKET_KEY *key;
+    TLS_TICKET_KEY tmp;
+    unsigned char *name;
+    time_t  now = time((time_t *) 0);
+
+    /* In tlsmgr requests we encode null key names as empty strings. */
+    name = LEN(buffer) ? (unsigned char *) STR(buffer) : 0;
+
+    /*
+     * Each key's encrypt and subsequent decrypt-only timeout is half of the
+     * total session timeout.
+     */
+    timeout /= 2;
+
+    /* Attempt to locate existing key */
+    if ((key = tls_scache_key(name, now, timeout)) == 0) {
+	if (name == 0) {
+	    /* Create new encryption key */
+	    if (RAND_bytes(tmp.name, TLS_TICKET_NAMELEN) <= 0
+		|| RAND_bytes(tmp.bits, TLS_TICKET_KEYLEN) <= 0
+		|| RAND_bytes(tmp.hmac, TLS_TICKET_MACLEN) <= 0)
+		return (TLS_MGR_STAT_ERR);
+	    tmp.tout = now + timeout - 1;
+	    key = tls_scache_key_rotate(&tmp);
+	} else {
+	    /* No matching decryption key found */
+	    return (TLS_MGR_STAT_ERR);
+	}
+    }
+    /* Return value overrites name buffer */
+    vstring_memcpy(buffer, (char *) key, sizeof(*key));
+    return (TLS_MGR_STAT_OK);
+}
+
 /* tlsmgr_loop - TLS manager main loop */
 
 static int tlsmgr_loop(char *unused_name, char **unused_argv)
@@ -661,6 +709,31 @@ static void tlsmgr_service(VSTREAM *client_stream, char *unused_service,
 		       ATTR_TYPE_END);
 	}
 
+	/*
+	 * RFC 5077 TLS session ticket keys
+	 */
+	else if (STREQ(STR(request), TLS_MGR_REQ_TKTKEY)) {
+	    if (attr_scan(client_stream, ATTR_FLAG_STRICT,
+			  ATTR_TYPE_DATA, TLS_MGR_ATTR_KEYNAME, buffer,
+			  ATTR_TYPE_END) == 1) {
+		if (LEN(buffer) != 0 && LEN(buffer) != TLS_TICKET_NAMELEN) {
+		    msg_warn("invalid session ticket key name length: %ld",
+			     (long) LEN(buffer));
+		    VSTRING_RESET(buffer);
+		} else if (*smtpd_cache.cache_timeout <= 0) {
+		    status = TLS_MGR_STAT_ERR;
+		    VSTRING_RESET(buffer);
+		} else {
+		    status = tlsmgr_key(buffer, *smtpd_cache.cache_timeout);
+		}
+	    }
+	    attr_print(client_stream, ATTR_FLAG_NONE,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
+		       ATTR_TYPE_DATA, TLS_MGR_ATTR_KEYBUF,
+		       LEN(buffer), STR(buffer),
+		       ATTR_TYPE_END);
+	}
+
 	/*
 	 * Entropy request.
 	 */
@@ -692,6 +765,7 @@ static void tlsmgr_service(VSTREAM *client_stream, char *unused_service,
 	 */
 	else if (STREQ(STR(request), TLS_MGR_REQ_POLICY)) {
 	    int     cachable = 0;
+	    int     timeout = 0;
 
 	    if (attr_scan(client_stream, ATTR_FLAG_STRICT,
 			  ATTR_TYPE_STR, TLS_MGR_ATTR_CACHE_TYPE, cache_type,
@@ -704,12 +778,14 @@ static void tlsmgr_service(VSTREAM *client_stream, char *unused_service,
 			     STR(cache_type), TLS_MGR_REQ_POLICY);
 		} else {
 		    cachable = (ent->cache_info != 0) ? 1 : 0;
+		    timeout = *ent->cache_timeout;
 		    status = TLS_MGR_STAT_OK;
 		}
 	    }
 	    attr_print(client_stream, ATTR_FLAG_NONE,
 		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
 		       ATTR_TYPE_INT, TLS_MGR_ATTR_CACHABLE, cachable,
+		       ATTR_TYPE_INT, TLS_MGR_ATTR_SESSTOUT, timeout,
 		       ATTR_TYPE_END);
 	}
 
@@ -841,7 +917,15 @@ static void tlsmgr_pre_init(char *unused_name, char **unused_argv)
      */
     dup_filter = htable_create(sizeof(cache_table) / sizeof(cache_table[0]));
     for (ent = cache_table; ent->cache_label; ++ent) {
-	if (**ent->cache_db) {
+	/* Sanitize session timeout */
+	if (*ent->cache_timeout > 0) {
+	    if (*ent->cache_timeout < TLS_SESSION_LIFEMIN)
+		*ent->cache_timeout = TLS_SESSION_LIFEMIN;
+	} else {
+	    *ent->cache_timeout = 0;
+	}
+	/* External cache database disabled if timeout is non-positive */
+	if (*ent->cache_timeout > 0 && **ent->cache_db) {
 	    if ((dup_label = htable_find(dup_filter, *ent->cache_db)) != 0)
 		msg_fatal("do not use the same TLS cache file %s for %s and %s",
 			  *ent->cache_db, dup_label, ent->cache_label);
@@ -849,7 +933,8 @@ static void tlsmgr_pre_init(char *unused_name, char **unused_argv)
 	    ent->cache_info =
 		tls_scache_open(data_redirect_map(redirect, *ent->cache_db),
 				ent->cache_label,
-				*ent->cache_loglevel >= 2,
+				tls_log_mask(ent->log_param,
+					   *ent->log_level) & TLS_LOG_CACHE,
 				*ent->cache_timeout);
 	}
     }
@@ -933,21 +1018,21 @@ int     main(int argc, char **argv)
 	VAR_SMTPD_TLS_SCACHE_DB, DEF_SMTPD_TLS_SCACHE_DB, &var_smtpd_tls_scache_db, 0, 0,
 	VAR_SMTP_TLS_SCACHE_DB, DEF_SMTP_TLS_SCACHE_DB, &var_smtp_tls_scache_db, 0, 0,
 	VAR_LMTP_TLS_SCACHE_DB, DEF_LMTP_TLS_SCACHE_DB, &var_lmtp_tls_scache_db, 0, 0,
+	VAR_SMTPD_TLS_LOGLEVEL, DEF_SMTPD_TLS_LOGLEVEL, &var_smtpd_tls_loglevel, 0, 0,
+	VAR_SMTP_TLS_LOGLEVEL, DEF_SMTP_TLS_LOGLEVEL, &var_smtp_tls_loglevel, 0, 0,
+	VAR_LMTP_TLS_LOGLEVEL, DEF_LMTP_TLS_LOGLEVEL, &var_lmtp_tls_loglevel, 0, 0,
 	0,
     };
     static const CONFIG_TIME_TABLE time_table[] = {
 	VAR_TLS_RESEED_PERIOD, DEF_TLS_RESEED_PERIOD, &var_tls_reseed_period, 1, 0,
 	VAR_TLS_PRNG_UPD_PERIOD, DEF_TLS_PRNG_UPD_PERIOD, &var_tls_prng_exch_period, 1, 0,
-	VAR_SMTPD_TLS_SCACHTIME, DEF_SMTPD_TLS_SCACHTIME, &var_smtpd_tls_scache_timeout, 0, 0,
-	VAR_SMTP_TLS_SCACHTIME, DEF_SMTP_TLS_SCACHTIME, &var_smtp_tls_scache_timeout, 0, 0,
-	VAR_LMTP_TLS_SCACHTIME, DEF_LMTP_TLS_SCACHTIME, &var_lmtp_tls_scache_timeout, 0, 0,
+	VAR_SMTPD_TLS_SCACHTIME, DEF_SMTPD_TLS_SCACHTIME, &var_smtpd_tls_scache_timeout, 0, MAX_SMTPD_TLS_SCACHETIME,
+	VAR_SMTP_TLS_SCACHTIME, DEF_SMTP_TLS_SCACHTIME, &var_smtp_tls_scache_timeout, 0, MAX_SMTP_TLS_SCACHETIME,
+	VAR_LMTP_TLS_SCACHTIME, DEF_LMTP_TLS_SCACHTIME, &var_lmtp_tls_scache_timeout, 0, MAX_LMTP_TLS_SCACHETIME,
 	0,
     };
     static const CONFIG_INT_TABLE int_table[] = {
 	VAR_TLS_RAND_BYTES, DEF_TLS_RAND_BYTES, &var_tls_rand_bytes, 1, 0,
-	VAR_SMTPD_TLS_LOGLEVEL, DEF_SMTPD_TLS_LOGLEVEL, &var_smtpd_tls_loglevel, 0, 0,
-	VAR_SMTP_TLS_LOGLEVEL, DEF_SMTP_TLS_LOGLEVEL, &var_smtp_tls_loglevel, 0, 0,
-	VAR_LMTP_TLS_LOGLEVEL, DEF_LMTP_TLS_LOGLEVEL, &var_lmtp_tls_loglevel, 0, 0,
 	0,
     };
 
