@@ -223,14 +223,14 @@ int     main(int unused_argc, char **argv)
     htable_enter(table, "bar-name", mystrdup("bar-value"));
     attr_print0(VSTREAM_OUT, ATTR_FLAG_NONE,
 		ATTR_TYPE_INT, ATTR_NAME_INT, 4711,
-		ATTR_TYPE_LONG, ATTR_NAME_LONG, 1234,
+		ATTR_TYPE_LONG, ATTR_NAME_LONG, 1234L,
 		ATTR_TYPE_STR, ATTR_NAME_STR, "whoopee",
 		ATTR_TYPE_DATA, ATTR_NAME_DATA, strlen("whoopee"), "whoopee",
 		ATTR_TYPE_HASH, table,
 		ATTR_TYPE_END);
     attr_print0(VSTREAM_OUT, ATTR_FLAG_NONE,
 		ATTR_TYPE_INT, ATTR_NAME_INT, 4711,
-		ATTR_TYPE_LONG, ATTR_NAME_LONG, 1234,
+		ATTR_TYPE_LONG, ATTR_NAME_LONG, 1234L,
 		ATTR_TYPE_STR, ATTR_NAME_STR, "whoopee",
 		ATTR_TYPE_DATA, ATTR_NAME_DATA, strlen("whoopee"), "whoopee",
 		ATTR_TYPE_END);
