@@ -15,6 +15,7 @@
   * Utility library.
   */
 #include <iostuff.h>
+#include <htable.h>
 
  /*
   * Listener external interface.
@@ -24,15 +25,13 @@ extern int inet_listen(const char *, int, int);
 extern int fifo_listen(const char *, int, int);
 extern int stream_listen(const char *, int, int);
 
-#define unix_pass_listen	unix_listen
-#define stream_pass_listen	stream_listen
-
 extern int inet_accept(int);
 extern int unix_accept(int);
 extern int stream_accept(int);
-extern int unix_pass_accept(int);
 
-#define stream_pass_accept	stream_accept
+extern int recv_pass_attr(int, HTABLE **, int, ssize_t);
+extern int pass_accept(int);
+extern int pass_accept_attr(int, HTABLE **);
 
 /* LICENSE
 /* .ad
