@@ -73,6 +73,12 @@ int     stream_connect(const char *path, int block_mode, int unused_timeout)
     if ((fifo = open(path, O_WRONLY | O_NONBLOCK, 0)) < 0)
 	return (-1);
 
+    /*
+     * This is for {unix,inet}_connect() compatibility.
+     */
+    if (block_mode == BLOCKING)
+	non_blocking(fifo, BLOCKING);
+
     /*
      * Create a pipe, and send one pipe end to the server.
      */
