@@ -132,7 +132,7 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
 		     VAR_STRICT_MBOX_OWNER);
 	} else {
 	    if (vstream_fseek(mp->fp, (off_t) 0, SEEK_END) < 0)
-		msg_fatal("%s: seek queue file %s: %m",
+		msg_fatal("%s: seek mailbox file %s: %m",
 			  myname, VSTREAM_PATH(mp->fp));
 	    mail_copy_status = mail_copy(COPY_ATTR(state.msg_attr), mp->fp,
 					 copy_flags, "\n", why);
